@@ -21,10 +21,8 @@
 """
 Bazaar support for versiontools
 """
-
-import bzrlib
-import bzrlib.errors
-import bzrlib.branch
+import logging
+import sys
 
 
 class BzrIntegration(object):
@@ -60,15 +58,25 @@ class BzrIntegration(object):
         """
         branch = None
         try:
+            import bzrlib
             if bzrlib.__version__ >= (2, 2, 1):
-                with bzrlib.initialize():
-                    branch = bzrlib.branch.Branch.open_containing(source_tree)[0]
+                # Python 2.4 the with keyword is not supported
+                # and so you need to use the context manager manually, sigh.
+                library_state = bzrlib.initialize()
+                library_state.__enter__()
+                try:
+                    from bzrlib import branch
+                    branch = branch.Branch.open_containing(source_tree)[0]
+                finally:
+                    library_state.__exit__(None, None, None)
             else:
-                branch = bzrlib.branch.Branch.open_containing(source_tree)[0]
-        except bzrlib.errors.NotBranchError:
-            import logging
-            logging.debug("Unable to get branch revision because"
-                            " directory %r is not a bzr branch",
-                            source_tree)
+                from bzrlib import branch
+                branch = branch.Branch.open_containing(source_tree)[0]
+        except Exception:
+            from versiontools import get_exception_message
+            message = get_exception_message(*sys.exc_info())
+            logging.debug("Unable to get branch revision because "
+                          "directory %r is not a bzr branch. Erorr: %s",
+                          (source_tree, message))
         if branch:
             return cls(branch)
